/*
 * single linked list
 *
 * Grigoriy Sitkarev, <sitkarev@unixkomi.ru>
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "list.h"
#include "tree.h"
#include "log.h"
#include "xmalloc.h"
#include "vec.h"
#include "buffer.h"
#include "str.h"
#include "par.h"
#include "pimage.h"

slist_t slist_new()
{
	slist_t s;

	s = xmalloc0(sizeof(*s));
	s->data = NULL;
	s->next = NULL;

	return s;
}

void slist_free(slist_t *s)
{
	if (!s || *s == NULL)
		error("slist_free: s NULL");

	xfree(*s);
	*s = NULL;
}

void slist_free_all(slist_t *slist)
{
	slist_t iter, next;

	if (!slist || *slist == NULL)
		error("slist_free_all: slist NULL");

	iter = *slist;
	while (iter) {
		next = iter->next;
		xfree(iter);
		iter = next;
	}
	*slist = NULL;
}

slist_t slist_add(slist_t slist, slist_t e)
{
	if (!e)
		error("slist_add: e NULL");

	if (e->next != NULL)
		warn("slist_add: element in list");

	if (slist)
		e->next = slist;
	slist = e;

	return slist;
}

slist_t slist_add_tail(slist_t slist, slist_t e)
{
	slist_t iter;

	if (!e)
		error("slist_add_tail: e NULL");

	if (e->next != NULL)
		warn("slist_add_tail: element in list");

	if (slist) {
		for (iter = slist; iter->next != NULL; iter = iter->next);
		iter->next = e;
		e->next = NULL;
	} else {
		slist = e;
		e->next = NULL;
	}

	return slist;
}

slist_t slist_extract(slist_t *slist)
{
	slist_t s = NULL;

	if (!slist)
		error("slist_remove: slist NULL");

	if (*slist) {
		s = *slist;
		*slist = (*slist)->next;
		s->next = NULL;
	}

	return s;
}

slist_t slist_unlink(slist_t slist, slist_t s)
{
	slist_t iter, prev = NULL;

	iter = slist;
	while (iter) {
		if (iter == s) {
			if (prev)
				prev->next = iter->next;
			else
				slist = iter->next;
			iter->next = NULL;
			return slist;
		}
		prev = iter;
		iter = iter->next;
	}

	return slist;
}

slist_t slist_reverse(slist_t slist)
{
	slist_t iter, reverse = NULL;

	iter = slist;
	while (iter) {
		slist = iter->next;
		if (reverse)
			iter->next = reverse;
		else
			iter->next = NULL;
		reverse = iter;
		iter = slist;
	}

	return reverse;
}

size_t slist_count(slist_t slist)
{
	size_t count = 0;
	slist_t iter;

	iter = slist;
	while (iter) {
		iter = iter->next;
		count++;
	}

	return count;
}

slist_t slist_find(slist_t slist, const void *data,
		int (*compare)(const void *a, const void *b))
{
	slist_t iter;

	for (iter = slist; iter != NULL; iter = iter->next) {
		if (compare) {
			if ((*compare)(iter->data, data) == 0)
				return iter;
		} else {
			if (iter->data == data)
				return iter;
		}
	}

	return NULL;
}

void slist_traverse(slist_t slist, slistcallback_t callback, void *opaque)
{
	int res;
	slist_t iter;

	for (iter = slist; iter != NULL; iter = iter->next) {

		res = callback(iter->data, opaque);

		if (res)
			return;
	}
}

/*
int main(int argc, char *argv[])
{
	int i;
	par_value_t pval;
	str_t str;
	struct par_image_struct *pi;
	struct par_struct *di_par[4];
	struct par_struct *ai_par[4];
	struct par_struct *eu_par[4];

	log_init(argv[0], LOG_LEVEL_DEBUG, 0, LOG_FACILITY_DAEMON, NULL);

	str = str_new();
	par_tree_create();
	station_add(0, "station0", 16, 32);
	station_add(1, "station1", 16, 32);
	station_add(2, "station2", 16, 32);
	pi = par_image_new(256, 256);
	par_container_add("/", "hw");
	par_container_add("/hw", "station0");

	for (i = 0; i < 4; i++) {
		str_snprintf(str, "AI_%d", i);
		par_parameter_add(&ai_par[i], "/hw/station0", str_ptr(str), PAR_INT);
		par_attach_to_station(ai_par[i], 0, PAR_DIR_IN, 0, i);
		par_image_add_par(pi, ai_par[i], PAR_DIR_IN, i*2, 0);
	}
	for (i = 0; i < 4; i++) {
		str_snprintf(str, "DI_%d", i);
		par_parameter_add(&di_par[i], "/hw/station0", str_ptr(str), PAR_BOOL);
		par_attach_to_station(di_par[i], 0, PAR_DIR_IN, 2, i);
		par_image_add_par(pi, di_par[i], PAR_DIR_OUT, i/8, i%8);
	}
	for (i = 0; i < 4; i++) {
		str_snprintf(str, "FL_%d", i);
		par_parameter_add(&eu_par[i], "/hw/station0", str_ptr(str), PAR_FLOAT);
		par_attach_to_station(eu_par[i], 0, PAR_DIR_OUT, 3, i+256);
		par_image_add_par(pi, eu_par[i], PAR_DIR_OUT, 8+i*4, 0);
	}

	par_image_update_inputs(pi);

	for (i = 0; i < 4; i++) {

		pval = par_read(ai_par[i]);
		printf("AI%d=%d\n", i, pval.value_int);
		if (i == 2)
			pval.value_bool = PAR_BOOL_ON;
		par_write(di_par[i], pval);
		pval.value_float = 2.718281828459045;
		par_write(eu_par[i], pval);
	}

	par_image_update_outputs(pi);
	par_image_dump(pi);
	par_tree_list();
	station_list();

	return 0;
}*/
